@@ -24,6 +24,9 @@ options:
     choices:
       - present
       - absent
+      - mounted
+      - unmounted
+      - quadlet
   recreate:
     description:
       - Recreate volume even if exists.
@@ -62,6 +65,38 @@ options:
       - Return additional information which can be helpful for investigations.
     type: bool
     default: False
+  quadlet_dir:
+    description:
+      - Path to the directory to write quadlet file in.
+        By default, it will be set as C(/etc/containers/systemd/) for root user,
+        C(~/.config/containers/systemd/) for non-root users.
+    type: path
+    required: false
+  quadlet_filename:
+    description:
+      - Name of quadlet file to write. By default it takes I(name) value.
+    type: str
+  quadlet_file_mode:
+    description:
+      - The permissions of the quadlet file.
+      - The O(quadlet_file_mode) can be specied as octal numbers or as a symbolic mode (for example, V(u+rwx) or V(u=rw,g=r,o=r)).
+        For octal numbers format, you must either add a leading zero so that Ansible's YAML parser knows it is an
+        octal number (like V(0644) or V(01777)) or quote it (like V('644') or V('1777')) so Ansible receives a string
+        and can do its own conversion from string into number. Giving Ansible a number without following one of these
+        rules will end up with a decimal number which will have unexpected results.
+      - If O(quadlet_file_mode) is not specified and the quadlet file B(does not) exist, the default V('0640') mask will be used
+        when setting the mode for the newly created file.
+      - If O(quadlet_file_mode) is not specified and the quadlet file B(does) exist, the mode of the existing file will be used.
+      - Specifying O(quadlet_file_mode) is the best way to ensure files are created with the correct permissions.
+    type: raw
+    required: false
+  quadlet_options:
+    description:
+      - Options for the quadlet file. Provide missing in usual network args
+        options as a list of lines to add.
+    type: list
+    elements: str
+    required: false
 
 requirements:
   - "podman"
@@ -74,13 +109,15 @@ volume:
   returned: always
   type: dict
   sample:
-    CreatedAt: '2020-06-05T16:38:55.277628769+03:00'
+    CreatedAt: '2023-11-30T16:41:31.310865559+02:00'
     Driver: local
-    Labels:
-      key.com: value
-      key.org: value2
-    Mountpoint: /home/user/.local/share/containers/storage/volumes/test/_data
-    Name: test
+    Labels: {}
+    LockNumber: 18
+    MountCount: 0
+    Mountpoint: /home/user/.local/share/containers/storage/volumes/volname/_data
+    Name: volname
+    NeedsChown: true
+    NeedsCopyUp: true
     Options: {}
     Scope: local
 
@@ -88,7 +125,8 @@ volume:
 
 EXAMPLES = '''
 # What modules does for example
-- podman_volume:
+- name: Create a volume
+  containers.podman.podman_volume:
     state: present
     name: volume1
     label:
@@ -97,14 +135,28 @@ EXAMPLES = '''
     options:
       - "device=/dev/loop1"
       - "type=ext4"
+
+- name: Create a Quadlet file for a volume
+  containers.podman.podman_volume:
+    state: quadlet
+    name: quadlet_volume
+    quadlet_filename: custom-name
+    quadlet_file_mode: '0640'
+    quadlet_options:
+      - Group=192
+      - Copy=true
+      - Image=quay.io/centos/centos:latest
+
 '''
 # noqa: F402
 import json  # noqa: F402
+import os  # noqa: F402
 
 from ansible.module_utils.basic import AnsibleModule  # noqa: F402
 from ansible.module_utils._text import to_bytes, to_native  # noqa: F402
 from ansible_collections.containers.podman.plugins.module_utils.podman.common import LooseVersion
 from ansible_collections.containers.podman.plugins.module_utils.podman.common import lower_keys
+from ansible_collections.containers.podman.plugins.module_utils.podman.quadlet import create_quadlet_state
 
 
 class PodmanVolumeModuleParams:
@@ -128,7 +180,7 @@ class PodmanVolumeModuleParams:
         Returns:
            list -- list of byte strings for Popen command
         """
-        if self.action in ['delete']:
+        if self.action in ['delete', 'mount', 'unmount']:
             return self._simple_action()
         if self.action in ['create']:
             return self._create_action()
@@ -137,6 +189,12 @@ class PodmanVolumeModuleParams:
         if self.action == 'delete':
             cmd = ['rm', '-f', self.params['name']]
             return [to_bytes(i, errors='surrogate_or_strict') for i in cmd]
+        if self.action == 'mount':
+            cmd = ['mount', self.params['name']]
+            return [to_bytes(i, errors='surrogate_or_strict') for i in cmd]
+        if self.action == 'unmount':
+            cmd = ['unmount', self.params['name']]
+            return [to_bytes(i, errors='surrogate_or_strict') for i in cmd]
 
     def _create_action(self):
         cmd = [self.action, self.params['name']]
@@ -294,6 +352,7 @@ class PodmanVolume:
         self.module = module
         self.name = name
         self.stdout, self.stderr = '', ''
+        self.mount_point = None
         self.info = self.get_info()
         self.version = self._get_podman_version()
         self.diff = {}
@@ -348,7 +407,7 @@ class PodmanVolume:
         """Perform action with volume.
 
         Arguments:
-            action {str} -- action to perform - create, stop, delete
+            action {str} -- action to perform - create, delete, mount, unmout
         """
         b_command = PodmanVolumeModuleParams(action,
                                              self.module.params,
@@ -357,11 +416,14 @@ class PodmanVolume:
                                              ).construct_command_from_params()
         full_cmd = " ".join([self.module.params['executable'], 'volume']
                             + [to_native(i) for i in b_command])
+        # check if running not from root
+        if os.getuid() != 0 and action == 'mount':
+            full_cmd = f"{self.module.params['executable']} unshare {full_cmd}"
         self.module.log("PODMAN-VOLUME-DEBUG: %s" % full_cmd)
         self.actions.append(full_cmd)
         if not self.module.check_mode:
             rc, out, err = self.module.run_command(
-                [self.module.params['executable'], b'volume'] + b_command,
+                full_cmd,
                 expand_user_and_vars=False)
             self.stdout = out
             self.stderr = err
@@ -369,6 +431,9 @@ class PodmanVolume:
                 self.module.fail_json(
                     msg="Can't %s volume %s" % (action, self.name),
                     stdout=out, stderr=err)
+            # in case of mount/unmount, return path to the volume from stdout
+            if action in ['mount']:
+                self.mount_point = out.strip()
 
     def delete(self):
         """Delete the volume."""
@@ -378,6 +443,14 @@ class PodmanVolume:
         """Create the volume."""
         self._perform_action('create')
 
+    def mount(self):
+        """Delete the volume."""
+        self._perform_action('mount')
+
+    def unmount(self):
+        """Create the volume."""
+        self._perform_action('unmount')
+
     def recreate(self):
         """Recreate the volume."""
         self.delete()
@@ -436,6 +509,9 @@ class PodmanVolumeManager:
         states_map = {
             'present': self.make_present,
             'absent': self.make_absent,
+            'mounted': self.make_mount,
+            'unmounted': self.make_unmount,
+            'quadlet': self.make_quadlet,
         }
         process_action = states_map[self.state]
         process_action()
@@ -468,12 +544,37 @@ class PodmanVolumeManager:
                              'podman_actions': self.volume.actions})
         self.module.exit_json(**self.results)
 
+    def make_mount(self):
+        """Run actions if desired state is 'mounted'."""
+        if not self.volume.exists:
+            self.volume.create()
+            self.results['actions'].append('created %s' % self.volume.name)
+        self.volume.mount()
+        self.results['actions'].append('mounted %s' % self.volume.name)
+        if self.volume.mount_point:
+            self.results.update({'mount_point': self.volume.mount_point})
+        self.update_volume_result()
+
+    def make_unmount(self):
+        """Run actions if desired state is 'unmounted'."""
+        if self.volume.exists:
+            self.volume.unmount()
+            self.results['actions'].append('unmounted %s' % self.volume.name)
+            self.update_volume_result()
+        else:
+            self.module.fail_json(msg="Volume %s does not exist!" % self.name)
+
+    def make_quadlet(self):
+        results_update = create_quadlet_state(self.module, "volume")
+        self.results.update(results_update)
+        self.module.exit_json(**self.results)
+
 
 def main():
     module = AnsibleModule(
         argument_spec=dict(
             state=dict(type='str', default="present",
-                       choices=['present', 'absent']),
+                       choices=['present', 'absent', 'mounted', 'unmounted', 'quadlet']),
             name=dict(type='str', required=True),
             label=dict(type='dict', required=False),
             driver=dict(type='str', required=False),
@@ -481,6 +582,10 @@ def main():
             recreate=dict(type='bool', default=False),
             executable=dict(type='str', required=False, default='podman'),
             debug=dict(type='bool', default=False),
+            quadlet_dir=dict(type='path', required=False),
+            quadlet_filename=dict(type='str', required=False),
+            quadlet_file_mode=dict(type='raw', required=False),
+            quadlet_options=dict(type='list', elements='str', required=False),
         ))
 
     PodmanVolumeManager(module).execute()
